// Functions for performing mathematical operations on single- and multi-dimensional arrays.

// Thomas Reppert 02/27/07

#include <math.h>

// Add2Arrays:
// Adds two arrays of the same length

void Add2Arrays(int Length, double* Array1, double* Array2, double* Final)
{
  int i;
	for(i = 0; i < Length; ++i)
	{
    Final[i] = Array1[i] + Array2[i];
	}
	return;
}

// Add4Arrays:
// Adds four arrays of the same length

void Add4Arrays(int Length, double* Array1, double* Array2, double* Array3, double* Array4, double* Final)
{
  int i;
	for(i = 0; i < Length; ++i)
	{
    Final[i] = Array1[i] + Array2[i] + Array3[i] + Array4[i];
	}
	return;
}

// Equal:
// Assigns all values of one array to another array

void Equal(int Length, double* Array1, double* Array2)
{
	int i;
	for(i = 0; i < Length; ++i)
	{
		Array1[i] = Array2[i];
	}
	return;
}

// Mag:
// Calculates the magnitude of a one-dimentional array

double Mag(int Length, double* Array)
{
	int i;
	double Final = 0.0;
	for(i = 0; i < Length; ++i)
	{
		Final = Final + Array[i]*Array[i];
	}
	Final = sqrt(Final);
	return Final;
}

// MinValue:
// Finds the minimum value and the corresponding index in the given array
// Note: Disregards the zeroth entry of the array

double MinValue(int Length, int* Index, double* Array)
{
	int i;
	double Min = 1.e10;
	for(i = 1; i < Length; ++i)
	{
		if(Array[i] < Min)
		{
			Min = Array[i];
			Index[1] = i;
		}
	}
	return Min;
}

// MtrxTmsVctr:
// Performs matrix multiplication between a multi-dimensional array and a single-dimensional array
// Note: Does not multiply the zeroth entries of the arrays

void MtrxTmsVctr(int Rows, int Cols, double** A, double* b, double* Final)
{
  int i, j;
	for(i = 1; i < Rows; ++i)
  {
    Final[i] = 0.0;
    for(j = 1; j < Cols; ++j)
    {
      Final[i] += A[i][j] * b[j];
    }
  }
  return;
}

// ScaleArray:
// Multiplies each component of an array by a scalar factor

void ScaleArray(int Length, double Scalar, double* Array1, double* Array2)
{
  int i;
  for(i = 0; i < Length; ++i)
  {
    Array2[i] = Array1[i] * Scalar;
  }
  return;
}
